﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Infragistics.Controls.Maps;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace CustomStyle
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const int MaxNodeCount = 200;
        readonly Style _capStyle1;
        readonly Style _capStyle2;
        private readonly Style _customNodeStyle2;

        public MainWindow()
        {
            InitializeComponent();

            _capStyle1 = Resources["CapStyle1"] as Style;
            _capStyle2 = Resources["CapStyle2"] as Style;
            _customNodeStyle2 = Resources["CustomNodeStyle2"] as Style;

            xnn.GlobalNodeLayouts.Add(new NetworkNodeNodeLayout
            {
                TargetTypeName = "NodeModel",
                DisplayMemberPath = "Label",
                ToolTipMemberPath = "ToolTip",
                ConnectionsMemberPath = "Connections",
                ConnectionTargetMemberPath = "Target",
            });

            xnn.LineEndCap = LineCap.Diamond;
            xnn.LineStartCap = LineCap.Ellipse;
            xnn.NodeControlAttachedEvent += xnn_NodeControlAttachedEvent;
            xnn.ItemsSource = new SampleTestData(MaxNodeCount).Nodes;

            this.KeyDown += MainWindow_KeyDown;
        }

        void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.OemPeriod:
                    SelectNextNode();
                    xnn.Focus();
                    break;
                case Key.OemComma:
                    SelectPreviousNode();
                    xnn.Focus();
                    break;
            }
        }

        void xnn_NodeControlAttachedEvent(object sender, NetworkNodeEventArgs e)
        {
            NetworkNodeNode node = e.NodeControl.Node;
            var nm = (NodeModel) node.Data;

            if(Convert.ToInt32(nm.ToolTip) % 2 == 0)
                node.Style = _customNodeStyle2;

            foreach (var connection in node.OutgoingConnections)
            {
                connection.LineEndCap = LineCap.Diamond;
                connection.LineStartCap = LineCap.Ellipse;
                connection.LineStartCapStyle = _capStyle1;
                connection.LineEndCapStyle = _capStyle2;
            }
        }

        private void ArrangeInRow_Click(object sender, RoutedEventArgs e)
        {
            List<NetworkNodeNode> nodesList = xnn.Nodes.ToList();
            List<Point> nodeLocations = GetNodeLocations();
            for(int i=0; i<MaxNodeCount; i++)
            {
                nodesList[i].Location = nodeLocations[i];
            }
            xnn.ScaleToFit();
        }

        private List<Point> GetNodeLocations()
        {
            var locsList = new List<Point>();
            bool checkEnd = true;
            var currentLoc = new Point(100.0, 100.0);
            locsList.Add(currentLoc);
            for (int i = 1; i <= MaxNodeCount; i++)
            {
                Point nextLoc;

                if (checkEnd)
                {
                    if ((currentLoc.X + MaxNodeCount) >= xnn.ActualWidth)
                    {
                        nextLoc = new Point(currentLoc.X, currentLoc.Y + MaxNodeCount);
                        checkEnd = false;
                    }
                    else
                    {
                        nextLoc = new Point(currentLoc.X + MaxNodeCount, currentLoc.Y);
                    }
                    locsList.Add(nextLoc);
                }
                else
                {
                    if ((currentLoc.X - MaxNodeCount) <= 0)
                    {
                        nextLoc = new Point(currentLoc.X, currentLoc.Y + MaxNodeCount);
                        checkEnd = true;
                    }
                    else
                    {
                        nextLoc = new Point(currentLoc.X - MaxNodeCount, currentLoc.Y);
                    }
                    locsList.Add(nextLoc);
                }
                currentLoc = nextLoc;
            }

            return locsList;
        }
        

        private void Previous_Click(object sender, RoutedEventArgs e)
        {
            SelectPreviousNode();
        }

        private void SelectPreviousNode()
        {
            if (xnn.SelectedNodes.Count > 0)
            {
                NetworkNodeNode selectedNode = xnn.SelectedNodes[0];

                if (selectedNode.IncomingConnections != null && selectedNode.IncomingConnections.ToList().Count > 0)
                {
                    NetworkNodeNode previousNode = selectedNode.IncomingConnections.First().Source;
                    xnn.SelectedNodes.Clear();
                    previousNode.IsSelected = true;
                    BringSelectedNodeIntoView();
                }
            }
        }

        private void Next_Click(object sender, RoutedEventArgs e)
        {
            SelectNextNode();
        }

        private void SelectNextNode()
        {
            if (xnn.SelectedNodes.Count > 0)
            {
                NetworkNodeNode selectedNode = xnn.SelectedNodes[0];

                if (selectedNode.OutgoingConnections != null && selectedNode.OutgoingConnections.ToList().Count > 0)
                {
                    NetworkNodeNode nextNode = selectedNode.OutgoingConnections.First().Target;
                    xnn.SelectedNodes.Clear();
                    nextNode.IsSelected = true;
                    BringSelectedNodeIntoView();
                }
            }
        }

        private void AutoArrange_Click(object sender, RoutedEventArgs e)
        {
            xnn.UpdateNodeArrangement();
            xnn.ScaleToFit();
        }

        private void BringSelectedNodeIntoView()
        {
            NetworkNodeNode selectedNode = xnn.SelectedNodes[0];
            bool adjustWindowRectangle = false;
            double targetLeft = xnn.WindowRect.Left;
            double targetRight = xnn.WindowRect.Right;
            double targetTop = xnn.WindowRect.Top;
            double targetBottom = xnn.WindowRect.Bottom;

            if (selectedNode.Location.X < xnn.WindowRect.Left)
            {
                double diff = xnn.WindowRect.Left - selectedNode.Location.X;
                targetLeft = selectedNode.Location.X;
                targetRight = xnn.WindowRect.Right - diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.X + 120 > xnn.WindowRect.Right)
            {
                double diff = selectedNode.Location.X + 120 - xnn.WindowRect.Right;
                targetRight = selectedNode.Location.X + 120;
                targetLeft = xnn.WindowRect.Left + diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.Y < xnn.WindowRect.Top)
            {
                double diff = xnn.WindowRect.Top - selectedNode.Location.Y;
                targetTop = selectedNode.Location.Y;
                targetBottom = xnn.WindowRect.Bottom - diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.Y + 40 > xnn.WindowRect.Bottom)
            {
                double diff = selectedNode.Location.Y + 40 - xnn.WindowRect.Bottom;
                targetBottom = selectedNode.Location.Y + 40;
                targetTop = xnn.WindowRect.Top + diff;
                adjustWindowRectangle = true;
            }

            if (adjustWindowRectangle)
            {
                xnn.WindowRect = new Rect(new Point(targetLeft, targetTop), new Point(targetRight, targetBottom));
            }
        }
    }


    public class SampleTestData
    {
        public ObservableCollection<NodeModel> Nodes { get; set; }

        public SampleTestData(int nodeCount)
        {
            Nodes = new ObservableCollection<NodeModel>();
            for (int i = 1; i <= nodeCount; i++)
            {
                var nm = new NodeModel
                {
                    Label = "NODE" + i,
                    ToolTip = i.ToString(),
                    Connections = new ObservableCollection<ConnectionModel>(),
                };
                Nodes.Add(nm);
            }

            for (int i = 1; i < nodeCount; i++)
            {
                NodeModel source = Nodes[i-1];
                NodeModel target = Nodes[i];
                var connectionModel = new ConnectionModel { Target = target };
                {
                    source.Connections.Add(connectionModel);
                }
            }
        }
    }

    public class NodeModel : INotifyPropertyChanged
    {
        private string _label;
        public string Label
        {
            get { return _label; }
            set
            {
                if (value != _label)
                {
                    _label = value;
                    NotifyPropertyUpdated("Label");
                }
            }
        }

        private string _toolTip;
        public string ToolTip
        {
            get { return _toolTip; }
            set
            {
                if (value != _toolTip)
                {
                    _toolTip = value;
                    NotifyPropertyUpdated("ToolTip");
                }
            }
        }

        private ObservableCollection<ConnectionModel> _connections;
        public ObservableCollection<ConnectionModel> Connections
        {
            get { return _connections; }
            set
            {
                if (value != _connections)
                {
                    _connections = value;
                    NotifyPropertyUpdated("Connections");
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void NotifyPropertyUpdated(string propertyName)
        {
            var handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class ConnectionModel : INotifyPropertyChanged
    {
        private NodeModel _target;
        public NodeModel Target
        {
            get { return _target; }
            set
            {
                if (value != _target)
                {
                    _target = value;
                    NotifyPropertyUpdated("Target");
                }
            }
        }

        #region Implementation of INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void NotifyPropertyUpdated(string propertyName)
        {
            var handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }


}
